/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam,
                        The Netherlands.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/
/*
  $Id: area.c,v 1.2 2000/05/12 12:18:20 olivierp Exp $  
 */

/*
 * The version of se is written by Mark van den Brand.
 */
             

#include <stdlib.h>

#include "se.h"
#include "area.h"
#include "path.h"
#include "tree.h"

ATerm newLength(int length)
{
  return ATmake("length(<int>)", length);
}

int getLength(ATerm atLength)
{
  int length;

  if(ATmatch(atLength, "length(<int>)", &length)) {
    return length;
  }
  else {
    ATerror("Illegal length\n");
    return -1;
  }
}                

ATerm newStart(int start)
{
  return ATmake("start(<int>)", start);
}

ATerm newArea(ATerm start, ATerm length)
{
  return ATmake("area(<term>,<term>)", start, length);
}

ATerm getLengthOfArea(ATerm area)
{
  ATerm atLength;

  if(ATmatch(area, "area(<term>,<term>)", NULL, &atLength)) {
    return atLength;
  }
  else {
    ATerror("Illegal area\n");
    return NULL;
  }
}

int getIntLengthOfArea(ATerm area)
{
  int length;
  ATerm atLength = getLengthOfArea(area);

  if(ATmatch(atLength, "length(<int>)", &length)) {
    return length;
  }
  else {
    ATerror("Illegal length\n");
    return -1;
  }
}                

ATerm setLengthOfArea(ATerm area, ATerm newLength)
{
  ATerm atStart;
  ATerm atLength;

  if(ATmatch(area, "area(<term>,<term>)", &atStart, &atLength)) {
    return ATmake("area(<term>,<term>)", atStart, newLength);
  }
  else {
    ATerror("Illegal area\n");
    return area;
  }
}

ATerm setIntLengthOfArea(ATerm area, int newLength)
{
  ATerm atLength = ATmake("length(<int>)", newLength);
  return setLengthOfArea(area, atLength);
}                

ATerm getStartOfArea(ATerm area)
{
  ATerm atStart;

  if(ATmatch(area, "area(<term>,<term>)", &atStart, NULL)) {
    return atStart;
  }
  else {
    ATerror("Illegal area\n");
    return NULL;
  }
}

int getIntStartOfArea(ATerm area)
{
  int start;
  ATerm atStart = getStartOfArea(area);

  if(ATmatch(atStart, "start(<int>)", &start)) {
    return start;
  }
  else {
    ATerror("Illegal start\n");
    return -1;
  }
}                

ATerm setStartOfArea(ATerm area, ATerm newStart)
{
  ATerm atStart;
  ATerm atLength;

  if(ATmatch(area, "area(<term>,<term>)", &atStart, &atLength)) {
    return ATmake("area(<term>,<term>)", newStart, atLength);
  }
  else {
    ATerror("Illegal area\n");
    return area;
  }
}

ATerm setIntStartOfArea(ATerm area, int newStart)
{
  ATerm atStart = ATmake("start(<int>)", newStart);
  return setStartOfArea(area, atStart);
}                


int areaStart(ATerm path, ATerm tree)
{
  if(isLeafPath(path)) { 
    return 0;
  }
  if(ATgetType(tree) == AT_APPL) {      
    if(asfix_is_appl(tree) || asfix_is_list(tree)) {
        return areaStart(getTail(path),
                         (ATerm)ATgetArgument((ATermAppl)tree, 2));
    }
    else if(asfix_is_term(tree)) {
      ATerm leftWs = ATgetArgument(tree, 5);
      ATerm newPath = getTail(path);
      newPath = decreaseHead(newPath);
      return getTreeLength(leftWs) +
             areaStart(newPath,
                       ATgetArgument((ATermAppl)tree, 6));
    } 
  }
  else if(ATgetType(tree) == AT_LIST) {     
    ATerm firstElement = ATgetFirst((ATermList)tree);
    if(isHeadZero(path)) {
      return areaStart(path, firstElement);  
    }
    else {
      ATerm newPath = decreaseHead(path);
      int elementLength = getTreeLength(firstElement);
      ATermList restElements = ATgetNext((ATermList)tree);
      return elementLength +
             areaStart(newPath, (ATerm)restElements);
    }
  }
  else {
    ATerror("In areaStart: Tree type not supported\n");
    return 0;
  }  
}

ATerm createArea(ATerm path, ATerm tree)
{
  ATerm subTree = subtreeOfTreeGivenPath(tree, path);
  int subTreeLength = getTreeLength(subTree);
  ATerm atLength = newLength(subTreeLength);

  int start = areaStart(path, tree) + 1;
  ATerm atStart = newStart(start);
  return newArea(atStart, atLength);
}

ATbool inArea(ATerm area, int loc)
{
  int start = getIntStartOfArea(area);
  int length = getIntLengthOfArea(area);
  if ((start <= loc) && (loc <= (start+length))) {
    return ATtrue;
  }
  else {
    return ATfalse;
  }                   
}

ATbool afterArea(ATerm area, int loc)
{
  int start = getIntStartOfArea(area);
  int length = getIntLengthOfArea(area);
  if (start+length < loc) {
    return ATtrue;
  }
  else {
    return ATfalse;
  }                   
}
