/*{{{   file header */

/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam,
                        The Netherlands.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/
/*
  $Id: se.c,v 1.3 2000/05/15 14:30:08 olivierp Exp $
 */                                                          

/*
 * The version of se is written by Mark van den Brand. 
 */

/*}}}  */

/*{{{   includes */

#include <stdlib.h>

#include "se.h"
#include "editor.h"
#include "focus.h"

/*}}}  */
/*{{{   globals */

static char myversion[] = "0.0"; 

/*}}}  */

/*{{{  static ATbool isValidMove(char *move) */

static ATbool isValidMove(char *move)
{
  return 
    strcmp(move, MOVE_LEFT) == 0 || 
    strcmp(move, MOVE_RIGHT) == 0 || 
    strcmp(move, MOVE_UP) == 0 || 
    strcmp(move, MOVE_DOWN) == 0;
}


/*}}}  */

/*{{{  void rec_terminate(int cid, ATerm t) */

void rec_terminate(int cid, ATerm t)
{
  exit(0);
}                                   

/*}}}  */
/*{{{  ATerm newEditor(int cid, ATerm editorId, char *text) */

ATerm newEditor(int cid, ATerm editorId, char *text)
{
  ATerm focusList, editor;

  editor = newEditorInstanceGivenText(text);
  PutValue(editorInstances, editorId, editor);

  focusList = getFocuses(editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm insertChars(int cid, ATerm editorId, int location, char *text) */

ATerm insertChars(int cid, ATerm editorId, int location, char *text)
{
  ATerm focusList;
  ATerm editor = GetValue(editorInstances, editorId);
  
  editor = insertCharsAtLocation(editor, location, text);
  PutValue(editorInstances, editorId, editor);

  focusList = getFocuses(editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm deleteChars(int cid, ATerm editorId, int location) */

ATerm deleteChars(int cid, ATerm editorId, int location)
{
  ATerm focusList;
  ATerm editor = GetValue(editorInstances, editorId);
  
  editor = deleteCharsAtLocation(editor, location, 1);
  PutValue(editorInstances, editorId, editor);

  focusList = getFocuses(editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm replaceFocus(int cid, ATerm editorId, ATerm tree) */

ATerm replaceFocus(int cid, ATerm editorId, ATerm tree)
{
  ATerm focusList;
  ATerm editor = GetValue(editorInstances, editorId);

  editor = replaceFocusByTree(editor, tree);
  PutValue(editorInstances, editorId, editor);

  focusList = getFocuses(editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm nextFocus(int cid, ATerm editorId) */

ATerm nextFocus(int cid, ATerm editorId)
{
  ATerm focusList;
  ATerm editor = GetValue(editorInstances, editorId);

  focusList = getFocuses(editor);
  focusList = (ATerm)ATgetNext((ATermList)focusList);

  editor = putFocuses(editor, focusList);

  PutValue(editorInstances, editorId, editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm setMultipleFocus(int cid, ATerm editorId, ATerm focuses) */

ATerm setMultipleFocus(int cid, ATerm editorId, ATerm focuses)
{
  ATerm editor = GetValue(editorInstances, editorId);

  editor = putFocuses(editor, focuses);
  PutValue(editorInstances, editorId, editor);

  return ATmake("snd-value(<term>)", focuses);
}

/*}}}  */
/*{{{  ATerm setSingleFocus(int cid, ATerm editorId, ATerm focus) */

ATerm setSingleFocus(int cid, ATerm editorId, ATerm focus)
{
  ATermList focusList;
  ATerm editor = GetValue(editorInstances, editorId);

  focusList = (ATermList)getFocuses(editor);
  focusList = ATinsert(focusList, focus);

  editor = putFocuses(editor, (ATerm)focusList);
  PutValue(editorInstances, editorId, editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm moveFocus(int cid, ATerm editorId, char *text) */

ATerm moveFocus(int cid, ATerm editorId, char *text)
{
  ATerm newEditor, focusList;
  ATerm editor = GetValue(editorInstances, editorId);

  if (isValidMove(text)) {
    newEditor = moveFocusInEditor(editor, ATparse(text));
  } 
  else {
    ATerror("Illegal move requested: %s\n", text);
    newEditor = editor;
  }
  focusList = getFocuses(newEditor);
  
  PutValue(editorInstances, editorId, newEditor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm getFocus(int cid, ATerm editorId, int location) */

ATerm getFocus(int cid, ATerm editorId, int location)
{
  ATerm focus;
  ATerm editor = GetValue(editorInstances, editorId);
  ATerm editorTree = getTreeInEditor(editor);
  focus = createFocusGivenLocation(editorTree, location);

  return ATmake("snd-value(<term>)", focus);
}

/*}}}  */
/*{{{  ATerm makeTreeInvalid(int cid, ATerm editorId) */

ATerm makeTreeInvalid(int cid, ATerm editorId)
{
  ATerm focusList;
  ATerm editor = GetValue(editorInstances, editorId);
  
  editor = makeTreeInvalidInEditor(editor);
  PutValue(editorInstances, editorId, editor);

  focusList = getFocuses(editor);

  return ATmake("snd-value(<term>)", focusList);
}

/*}}}  */
/*{{{  ATerm getTree(int cid, ATerm editorId) */

ATerm getTree(int cid, ATerm editorId)
{
  ATerm editor = GetValue(editorInstances, editorId);
  ATerm tree = getTreeInEditor(editor);

  return ATmake("snd-value(<term>)", tree);
}

/*}}}  */
/*{{{  void usage(char *prg, ATbool is_err) */

void usage(char *prg, ATbool is_err)
{
    ATwarning("usage: %s [aterm-options] [toolbus-options]\n", prg);
    ATwarning("use '%s -at-help' to get more options.\n", prg);
    ATwarning("This program can only be used as a ToolBus tool!\n");
    exit(is_err ? 1 : 0);
}

/*}}}  */
/*{{{  void version(const char *msg) */

void version(const char *msg)
{
    ATwarning("%s v%s\n", msg, myversion);
    exit(1);
}   

/*}}}  */
/*{{{  int main(int argc, char *argv[]) */

int main(int argc, char *argv[])
{
  int i, cid;
  ATerm bottomOfStack;

  for (i=1; i<argc; i++) {
      if (strcmp(argv[i], "-h") == 0) {
          usage(argv[0], ATfalse);
      } else if (strcmp(argv[i], "-V") == 0) {
          version(argv[0]);
      }
  }

  ATBinit(argc, argv,&bottomOfStack);
  AFinit(argc, argv, &bottomOfStack);


  cid = ATBconnect(NULL, NULL, -1, se_handler);   

  editorInstances = CreateValueStore(100, 75);

  ATBeventloop();

  return 0;
}                                       

/*}}}  */
